@@ -92,10 +92,10 @@ can make up an address.
 
 The main  address formats  currently supported are as follows. (a ? means the component is optional):
 
-    'suburban' : sub_property_identifier(?) property_identifier(?) street street_type suburb subcountry post_code(?)country(?)
+    'suburban' : sub_property(?) property_identifier(?) street street_type suburb subcountry post_code(?)country(?)
 
     OR for the USA
-    'suburban' : property_identifier(?) street street_type sub_property_identifier(?) suburb subcountry post_code(?) country(?)
+    'suburban' : property_identifier(?) street street_type sub_property(?) suburb subcountry post_code(?) country(?)
 
     'rural'    : property_name suburb subcountry post_code(?) country(?)
     'post_box' : post_box suburb subcountry post_code(?) country(?)
@@ -153,7 +153,7 @@ country name is supplied that the module doesn't recognise, it will die.
 
 This option only applies to the C<case_all> method, see below.
 
-=item auto_clean  (optional)
+=item auto_clean (optional)
 
 When this option is set to a positive value, the input string is
 'cleaned' to try and normalise bad patterns. The type of cleaning
@@ -180,8 +180,7 @@ of sub country is allowed, such as "NSW" and not "New South Wales". This
 will make parsing quicker and ensure that addresses comply with postal
 standards that normally permit only abbreviated sub countries.
 
-It also avoids matching a sub_country name too early, as in the case of
-    'Port Washington New Jersey'
+It also avoids matching a sub_country name too early, as in the case of 'Port Washington New Jersey'
 Normally, 'Washington would be consumed as the sub country, but by first converting
 the address to 'Port Washington NJ' we avoid this problem
 
@@ -225,6 +224,7 @@ following keys are used for each component:
     po_box_type - such as 'Private Boxes'
     post_box
     road_box
+    sub_property_type
     sub_property_identifier
     property_identifier
     property_name
@@ -275,7 +275,7 @@ but you still want to title case what you have.
 
 =head2 properties
 
-The C<properties> method return several properties of the address as a hash.
+The C<properties> method returns several properties of the address as a hash.
 The  following keys are used for each property -
 
     type - either suburban ,rural,post_box,road_box,unknown
@@ -285,10 +285,14 @@ The  following keys are used for each property -
 Additional properties can be accessed with the following
 
   $address->{original_input}
-  $address->{input_string} - string after  auto_clean option has been applied
+  $address->{input_string} - string after auto_clean option has been applied
   $address->{country_code} - abbreviated Country address format (as defined in the C<new> method)
   $address->{error} - error flag, 0 = good, 1 = error
-  $address->{error_desc} - text to  to describe the type of parsing error
+  $address->{error_desc} - text to describe the type of parsing error
+  $address->{warning} - warning flag, 0 = good, 1 = warning
+  $address->{warning_desc} - text to to describe the type of parsing warning(s)
+  
+Warnings mean that the address has parsed but there may still be errors within it's components  
 
 
 =head2 report
@@ -302,6 +306,8 @@ Create a formatted text report
     any non matching part of input string
     if any parsing errors occurred
     error description
+    if any parsing warning occurred
+    warning description
 
     the name and value of each defined component
 
@@ -329,7 +335,7 @@ it is impossible to correctly identify them all.
 
 Valid addresses must contain:
 
-    property address, suburb,  subcountry (aka state) in that order.
+    property address, suburb, subcountry (aka state) in that order.
 
 This format is widely accepted in Australia and the US.
 
@@ -402,7 +408,7 @@ use Lingua::EN::AddressParse::Grammar;
 use Lingua::EN::NameParse;
 use Parse::RecDescent;
 
-our $VERSION = '1.22';
+our $VERSION = '1.23';
 
 #------------------------------------------------------------------------------
 # Create a new instance of an address parsing object. This step is time
@@ -462,7 +468,6 @@ sub parse
     ($po_box_type,$address->{input_string}) = _extract_po_box_type($address->{input_string});
 
 
-
     # Replace commas (which can be used to chunk sections of addresses) with spaces
     $address->{input_string} =~ s/,/ /g;
 
@@ -470,6 +475,12 @@ sub parse
     {
         $address->{input_string} = _clean($address);
     }
+    
+    # Normalise sub property ID, 4/22-24 => UNIT 4 22-24
+    if ($address->{country_code} ne 'US' and $address->{input_string} =~ /^(\d{1,4}[A-Z]{0,2})[\/| ](\d.*)$/ )
+    {
+        $address->{input_string} = "UNIT $1 $2";     
+    }    
 
     # We need to add a trailing space to the input string. This is because the grammar
     # tree expects a terminator (the space) fro every production, optionally followed
@@ -505,7 +516,7 @@ sub case_components
         my $cased_value;
         my $curr_value = $orig_components{$curr_key};
 
-        if ( $curr_key =~ /^(base_street_name|street_name|street_type|suburb|property_name|sub_property_identifier|pre_cursor|po_box_type)$/ )
+        if ( $curr_key =~ /^(base_street_name|street_name|street_type|suburb|property_name|sub_property|pre_cursor|po_box_type)/ )
         {
 
             if ( $curr_key eq 'street_name' and$curr_value =~ /^US HIGHWAY (.*)/ )
@@ -563,10 +574,14 @@ sub case_components
                 # N, SE etc is capitalised
                 $cased_value =$curr_value;
             }
-            else
+            elsif (length($curr_value) > 2)
             {
                 $cased_value = Lingua::EN::NameParse::case_surname($curr_value);
             }
+            else
+            {
+                $cased_value = '';
+            }
         }
         # retain sub country and countries capitalisation, as usually uppercase
         else
@@ -596,16 +611,16 @@ sub case_all
         (
            'rural'   => [ qw/pre_cursor property_name suburb subcountry post_code country/],
            'post_box'=> [ qw/pre_cursor post_box suburb po_box_type subcountry post_code country/ ],
-           'road_box'=> [ qw/pre_cursor road_box street street_type suburb subcountry post_code country/ ]
+           'road_box'=> [ qw/pre_cursor road_box street_name street_type suburb subcountry post_code country/ ]
 
         );
         if ( $address->{country} eq 'US' )
         {
-           $component_order{'suburban'} = [ qw/pre_cursor property_identifier street street_type street_direction_suffix sub_property_identifier suburb subcountry post_code country/];
+           $component_order{'suburban'} = [ qw/pre_cursor property_identifier street_name  street_type street_direction_suffix sub_property_type sub_property_identifier suburb subcountry post_code country/];
         }
         else
         {
-           $component_order{'suburban'} = [ qw/pre_cursor sub_property_identifier property_identifier street street_type suburb subcountry post_code country/ ];
+           $component_order{'suburban'} = [ qw/pre_cursor sub_property_type sub_property_identifier property_identifier street_name  street_type suburb subcountry post_code country/ ];
         }
 
         my %component_vals = $address->case_components;
@@ -666,6 +681,8 @@ sub report
     _fmt_report_line(\$report,"Non matching part",$props{non_matching});
     _fmt_report_line(\$report,"Error",$address->{error});
     _fmt_report_line(\$report,"Error descriptions",$address->{error_desc});
+    _fmt_report_line(\$report,"Warning",$address->{error});
+    _fmt_report_line(\$report,"Warning description",$address->{warning_desc});    
     _fmt_report_line(\$report,"Case all",$address->case_all);
 
 
@@ -721,38 +738,43 @@ sub _assemble
         $address->{components}{'po_box_type'} = '';
     }
 
-    # For correct matching, the grammar of each component must include the
-    # trailing space that separates it from any following word. This should
-    # now be removed from each component
+  
 
     $address->{components}{post_box} = '';
     if ( $parsed_address->{post_box} )
     {
-        $address->{components}{post_box} = _trim_trailing_space($parsed_address->{post_box});
+        $address->{components}{post_box} = $parsed_address->{post_box};
     }
 
     $address->{components}{road_box} = '';
     if ( $parsed_address->{road_box} )
     {
-        $address->{components}{road_box} = _trim_trailing_space($parsed_address->{road_box});
+        $address->{components}{road_box} = $parsed_address->{road_box};
     }
 
     $address->{components}{property_name} = '';
     if ( $parsed_address->{property_name} )
     {
-        $address->{components}{property_name} = _trim_trailing_space($parsed_address->{property_name});
+        $address->{components}{property_name} = $parsed_address->{property_name};
     }
 
     $address->{components}{sub_property_identifier} = '';
-    if ( $parsed_address->{sub_property_identifier} )
+    $address->{components}{sub_property_type} = '';
+    
+    if ( $parsed_address->{sub_property} )
     {
-        $address->{components}{sub_property_identifier} = _trim_trailing_space($parsed_address->{sub_property_identifier});
+        if ($parsed_address->{sub_property} =~ /^(#|\w{1,}) (.*)$/ )
+        {
+            # Such as Unit 24, # 4A etc
+            $address->{components}{sub_property_type} = $1;
+            $address->{components}{sub_property_identifier} = $2;
+        }
     }
 
     $address->{components}{property_identifier} = '';
     if ( $parsed_address->{property_identifier} )
     {
-        $address->{components}{property_identifier} = _trim_trailing_space($parsed_address->{property_identifier});
+        $address->{components}{property_identifier} = $parsed_address->{property_identifier};
     }
 
     $address->{components}{base_street_name} = '';
@@ -763,6 +785,10 @@ sub _assemble
         $address->{components}{street_direction_prefix} =  $street_direction;
         $address->{components}{base_street_name} = $base_street_name;
     }
+    else
+    {
+        $address->{components}{base_street_name} = $parsed_address->{street_name};
+    }
 
     $address->{components}{street_name} = '';
     $address->{components}{street_type} = '';
@@ -771,12 +797,12 @@ sub _assemble
         # Streets such as 'The Corso' will parse as street_name = 'The' and street_type = 'Corso', so seperate out
         if ( $parsed_address->{street_name} eq 'THE ' )
         {
-             $address->{components}{street_name} = 'THE ' .  _trim_trailing_space($parsed_address->{street_type});
+             $address->{components}{street_name} = 'THE ' . $parsed_address->{street_type};
         }
         else
         {
-            $address->{components}{street_name} = _trim_trailing_space($parsed_address->{street_name});
-            $address->{components}{street_type} =  _trim_trailing_space($parsed_address->{street_type});
+            $address->{components}{street_name} = $parsed_address->{street_name};
+            $address->{components}{street_type} = $parsed_address->{street_type};
         }
     }
 
@@ -784,20 +810,20 @@ sub _assemble
     $address->{components}{street_direction_suffix} = '';
     if ( $parsed_address->{street_direction_suffix} )
     {
-        $address->{components}{street_direction_suffix} =  _trim_trailing_space($parsed_address->{street_direction_suffix});
+        $address->{components}{street_direction_suffix} =  $parsed_address->{street_direction_suffix};
     }
 
 
     $address->{components}{suburb} = '';
     if ( $parsed_address->{suburb} )
     {
-        $address->{components}{suburb} =  _trim_trailing_space($parsed_address->{suburb});
+        $address->{components}{suburb} =  $parsed_address->{suburb};
     }
 
     $address->{components}{subcountry} = '';
     if ( $parsed_address->{subcountry} )
     {
-        my $sub_country = _trim_trailing_space($parsed_address->{subcountry});
+        my $sub_country = $parsed_address->{subcountry};
 
         # Force sub country to abbreviated form, South Australia becomes SA, Michigan become MI etc
         if ($address->{abbreviate_subcountry})
@@ -823,13 +849,13 @@ sub _assemble
     $address->{components}{post_code} = '';
     if ( $parsed_address->{post_code} )
     {
-        $address->{components}{post_code} = _trim_trailing_space($parsed_address->{post_code});
+        $address->{components}{post_code} = $parsed_address->{post_code};
     }
 
     $address->{components}{country} = '';
     if ( $parsed_address->{country} )
     {
-        $address->{components}{country} = _trim_trailing_space($parsed_address->{country});
+        $address->{components}{country} = $parsed_address->{country};
     }
 
     $address->{properties} = ();
@@ -840,7 +866,8 @@ sub _assemble
         $address->{properties}{non_matching} = $parsed_address->{non_matching};
     }
     $address->{properties}{type} = $parsed_address->{type};
-
+    
+    _trim_trailing_space($address);
 
     return($address);
 }
@@ -881,6 +908,8 @@ sub _validate
     my $address = shift;
     $address->{error} = 0;
     $address->{error_desc} = '';
+    $address->{warning} = 0;
+    $address->{warning_desc} = '';
 
     if ( $address->{properties}{non_matching} )
     {
@@ -913,22 +942,22 @@ sub _validate
             if ( $address->{properties}{type} eq 'suburban' )
             {
                 my $street = $address->{components}{street_name};
-                if ($street !~ /\d/ and length($street) > 1 )
+                if ($street !~ /\d/ )
                 {
                     # Not an ordinal or single letter street type
-                    if ( $address->{components}{street_name} !~ /[AEIOUY]/ )
+                    if ( _check_vowel($address->{components}{base_street_name}) )
                     {
                         # street name must have a vowel sound,
-                        $address->{error} = 1;
-                        $address->{error_desc} .= ':no vowel sound in street';
+                        $address->{warning} = 1;
+                        $address->{warning_desc} .= ";no vowel sound in street word : $address->{components}{base_street_name}";
                     }
                 }
             }
 
-            if ( $address->{components}{suburb} !~ /[AEIOUY]/ )
+            if ( _check_vowel($address->{components}{suburb}) )
             {
-                $address->{error} = 1;
-                $address->{error_desc} .= ':no vowel sound in suburb';
+                $address->{warning} = 1;
+                $address->{warning_desc} .= ";no vowel sound in suburb word : $address->{components}{suburb}";
             }
         }
     }
@@ -957,6 +986,12 @@ sub _clean
     $input =~ s|  +| |go ;
     $input =~ s|^ ||;
     $input =~ s| $||;
+    
+    # remove redundant padding around sub proprty and property IDs
+    $input =~ s| -|-|;
+    $input =~ s|- |-|;
+    $input =~ s| /|/|;
+    $input =~ s|/ |/|;
 
     # Expand abbreviations that are too short
 
@@ -1077,19 +1112,46 @@ sub _extract_po_box_type
     }
 }
 #------------------------------------------------------------------------------
-# Remove any trailing spaces
+# For correct matching, the grammar of each component must include the
+# trailing space that separates it from any following word. This should
+# now be removed from each component
 
 sub _trim_trailing_space
 {
-    my ($string) = @_;
-    $string =~ s/ $//;
-    return($string);
+    my ($address) = @_;
+    
+    foreach my $key (keys %{ $address->{components} } )
+    {
+        if ($address->{components}{$key} )
+        {
+            $address->{components}{$key}  =~ s/ $//;
+        }
+    }         
 }
 #------------------------------------------------------------------------------
 
 sub _fmt_report_line
 {
     my ($report_ref,$label,$value) = @_;
+    # To DO $$ ??
     $$report_ref .= sprintf("%-23.23s '%s'\n",$label,$value);
 }
+#------------------------------------------------------------------------------
+
+sub _check_vowel
+{
+    my ($str) = @_;
+
+    my @words = split(/ /,$str);
+    foreach my $word (@words)
+    {
+        if ( length($word) > 1 and $word !~ /[AEIOUY]|ST|MT/ )  # St is abbreviation for Saint
+        {
+            return(1);
+        }
+    }
+    return(0);
+}
+#------------------------------------------------------------------------------
+
 return(1);
