@@ -35,7 +35,7 @@ use strict;
 use warnings;
 use Locale::SubCountry;
 
-our $VERSION = '1.22';
+our $VERSION = '1.23';
 
 #-------------------------------------------------------------------------------
 # Rules that define valid orderings of an addresses components
@@ -51,7 +51,7 @@ q{
     # will allow for cases such as 'Lot 123 Xyz Street' where Lot is in effect the house number, even though 'Lot' is grouped as a sub_property label
     # Also, cases such as 'SHOP 12A, CHAPEL RD STH' have no street number so are incomplete, but still may need to be parsed
 
-    sub_property_identifier(?) property_identifier(?) street_untyped suburb subcountry post_code(?) country(?) non_matching(?)
+    sub_property(?) property_identifier(?) street_untyped suburb subcountry post_code(?) country(?) non_matching(?)
     {
         # block of code to define actions upon successful completion of a
         # 'production' or rule
@@ -60,7 +60,7 @@ q{
         {
             # Parse::RecDescent lets you return a single scalar, which we use as
             # an anonymous hash reference
-            sub_property_identifier => $item[1][0],
+            sub_property            => $item[1][0],
             property_identifier     => $item[2][0],
             street_name             => $item[3],
             street_type             => '',
@@ -74,11 +74,11 @@ q{
     }
     |
 
-    sub_property_identifier(?) property_identifier(?) street street_type suburb subcountry post_code(?) country(?) non_matching(?)
+    sub_property(?) property_identifier(?) street street_type suburb subcountry post_code(?) country(?) non_matching(?)
     {
         $return =
         {
-            sub_property_identifier => $item[1][0],
+            sub_property            => $item[1][0],
             property_identifier     => $item[2][0],
             street_name             => $item[3],
             street_type             => $item[4],
@@ -101,7 +101,7 @@ q{
     full_address :
 
 
-    property_identifier(?) street_untyped sub_property_identifier(?) suburb subcountry post_code(?) country(?) non_matching(?)
+    property_identifier(?) street_untyped sub_property(?) suburb subcountry post_code(?) country(?) non_matching(?)
     # (needs higher precedence than streets with types)
 
     {
@@ -110,7 +110,7 @@ q{
             property_identifier     => $item[1][0],
             street_name             => $item[2],
             street_type             => '',
-            sub_property_identifier => $item[3][0],
+            sub_property            =>  $item[3][0],
             suburb                  => $item[4],
             subcountry              => $item[5],
             post_code               => $item[6][0],
@@ -121,7 +121,7 @@ q{
     }
     |
 
-    property_identifier(?) street street_type abbrev_direction(?) sub_property_identifier(?) suburb subcountry post_code(?) country(?) non_matching(?)
+    property_identifier(?) street street_type abbrev_direction(?) sub_property(?) suburb subcountry post_code(?) country(?) non_matching(?)
     {
         $return =
         {
@@ -129,7 +129,7 @@ q{
             street_name             => $item[2],
             street_type             => $item[3],
             street_direction_suffix => $item[4][0],
-            sub_property_identifier => $item[5][0],
+            sub_property            => $item[5][0],
             suburb                  => $item[6],
             subcountry              => $item[7],
             post_code               => $item[8][0],
@@ -266,28 +266,29 @@ q{
 # expressed as literals or Perl regular expressions.
 #------------------------------------------------------------------------------
 
-my $sub_property_identifier =
+my $sub_property =
 q{
 
     # Level 4 Tower A,
     # Unit 3A Block C
-    sub_property_identifier :
+    sub_property:
+
         /LEVEL (\d+|[GM]) (BUILDING|TOWER) \d+[A-Z]? / |
         /LEVEL (\d+|[GM]) (BUILDING|TOWER) [A-Z] / |
         /UNIT \d+[A-Z]? (BLOCK|BLDG|BUILDING|TOWER) ([A-Z]|\d+) / |
         /\d{1,2}(ST|ND|RD|TH) (FLOOR|FLR|FL) / |
         /(FLOOR|FLR|FL) \d{1,2} /
-
         |
-
-        sub_property_name sub_property_number
+        sub_property_type unit_number
         {
            $return = "$item[1]$item[2]"
         }
+ 
+
 
     # Unit 34, BLDG 12, SUITE A,
 
-    sub_property_name:
+    sub_property_type:
         /(
         APARTMENT   | APT   |
         BAY         |
@@ -325,13 +326,12 @@ q{
         \#                # Note '#' is a common abbreviation for number in USA
         )\ /x
 
-    sub_property_number :
-        /\d{1,6}[A-Z]{0,2} / | # such as 23B, 6AW
+    unit_number:
+        /\d{1,6} /           |
+        /\d{1,4}[A-Z]{0,2} / | # such as 23B, 6AW
         /\d{1,2}[A-Z]\d /    | # such as 4A5
-        /[A-Z]{1,2}\d{0,4} / | # such as # D512 # changed for Erik
-        /\d{1,3}-\d{1,3} /     # such as 114-234. 12-33 can be building 1 floor 14  unit 234
-
-
+        /[A-Z]{1,2}\d{0,4} / | # such as # D512
+        /\d{1,3}-\d{1,3}/        
 };
 
 #------------------------------------------------------------------------------
@@ -340,14 +340,10 @@ my $property_identifier =
 q{
     property_identifier :
 
-        /\d{1,3}[A-Z]?[\/ ]\d{1,5}-\d{1,5} / |  # 12/42-44, 12 42-44
-        /\d{1,3}[A-Z]?[\/ ]\d{1,5}[A-Z]? /   |  # 12A/42, 12 42A
-        /\d{1,4} 1\/2 /                      |  # fractional number such as 22 1/2 (half numbers are valid in US)
-        /\d{1,5}-\d{1,5} /                   |  # 1002-1006
-        /\d{1,5}[A-Z]? /                        # 10025A
-
+        /\d{1,4} 1\/2 /    |  # fractional number such as 22 1/2 (half numbers are valid in US)
+        /\d{1,5}-\d{1,5} / |  # 1002-1006
+        /\d{1,5}[A-Z]? /      # 10025A
 };
-
 #------------------------------------------------------------------------------
 
 my $property_name =
@@ -400,8 +396,6 @@ q{
     road_box_number : /[A-Z]?\d{1,5}[A-Z]? /
 
 };
-
-
 #------------------------------------------------------------------------------
 
 my $street =
@@ -413,18 +407,15 @@ q{
         major_road |
         avenue_letter |
         street_name_single_word |
-        /RUE (DE |DES )?/ any_word
-        {
-            $return = "$item[1]$item[2]"
-        }
-        |
-        /AVENUE OF THE AMERICAS /
-
+        street_noun |
+        french_style |
+        /AVENUE OF \w+ (\w+ )?/  # The Americas, Two Roivers wtc
+        
     major_road :
-        /(COUNTY |STATE |US )?(ALT|HIGHWAY|HWY|ROAD|RD|ROUTE) \d{1,3} ([N|E|S|W|NORTH|EAST|SOUTH|WEST] )?/
+        /([N|E|S|W] )?(COUNTY |STATE |US |FIRE )?(ALT|HIGHWAY|LANE|HWY|ROAD|RD|ROUTE) \d{1,3}\w? ([N|E|S|W|NORTH|EAST|SOUTH|WEST] )?/
 
     avenue_letter :
-        /AVENUE [A-Z] /
+        /([N|E|S|W] )?AVENUE [A-Z] /
 
     street_name_single_word:
         /([N|E|S|W] )?BROADWAY|BOARDWALK|BOULEVARD|BOWERY|ESPLANADE|KINGSWAY|QUEENSWAY|GREENWAY|PARKWAY|RIVERBANK /
@@ -432,6 +423,18 @@ q{
         {
             $return = $item[1]
         }
+        
+    street_noun:
+        /THE / any_word
+        {
+            $return = "$item[1]$item[2]"
+        }
+        
+    french_style:    
+        /RUE (DE |DES )?/ any_word
+        {
+            $return = "$item[1]$item[2]"
+        }        
 
     #----------------------------------------------------------------------------
 
@@ -464,9 +467,10 @@ q{
         MOUNT|
         DAME|
         SIR|
-        U|UPPER|
-        L|LOWER|
-        LA
+        UPPER|
+        LOWER|
+        LA|
+        ST
         )\ /x
 
     street_name :
@@ -481,11 +485,16 @@ q{
         any_word
         /(
         BEND|
+        BRAE|
         BURN|
         CAY|
         CHASE|
+        CIRCLE|
         CLUB|
+        CREST|
+        CRESCENT|
         CROSS|
+        CROSSING|
         COVE|
         GARDEN|
         GLEN|
@@ -504,12 +513,15 @@ q{
         PATH|
         PARKWAY|
         PLACE|
+        PLAZA|
         POINT|
         POINTE|
         RUN|
         RIDGE|
         SQUARE|
+        TRAIL|
         VIEW|
+        VILLAGE|
         VISTA
         )\ /x
         ...street_type
@@ -521,19 +533,24 @@ q{
         # STREET_TYPE WORD STREET_TYPE
         # Glen Alpine Way, La Boheme Ave,  Grove Valley Ave, Green Bay Road
         /(
+        CIRCLE|
         CLUB|
         CRESCENT|
         CROSS|
+        GLADE|
         GLEN|
-        GREEN|
+        GREENS?|
         GROVE|
         FAIRWAY|
+        HOLLOW|
         HILL|
         KEY|
         LA|
+        LANE|
         LT|
         PARK|
         PLAZA|
+        RIDGE|
         ST
         )\ /x
         street_name_word ...street_type
@@ -541,6 +558,12 @@ q{
             $return = "$item[1]$item[2]"
         }
         |
+        # New York State has streets such as 'Dutch Street Road'
+        #any_word /STREET / .../ROAD|RD /
+        #{
+        #    $return = "$item[1]$item[2]"
+        #}
+        #|        
 
         # Allow for street_type that can also occur as a street name, eg Park Lane, Green Street
         any_word ...street_type
@@ -612,124 +635,103 @@ q{
         }
     }
 
-
     street_name_letter:  /[A-Z]\s+/  # eg B (Street)
 
     street_type:
 
         /(
         # Place most frequent types first to speed up matching
-        ST     |STREET         |
-        RD     |ROAD           |
-        LA     |LN | LANE      |
-        AVE?   | AVENUE        |
+        ST|STREET|
+        RD|ROAD|
+        LA|LN|LANE|
+        AVE?|AVENUE|
 
-        ALY?   | ALLEY         |
-        ARC    | ARCADE        |
-        BATTLEMENT |
-        BROADWATER |
-        BAYWAY    |
-        BVD    | BLVD?      | BOULEVARDE?   |
-        BND    | BEND          |
-        BL     | BOWL          |
-        BR     | BRAE          |
-        BROW      |
-        CASCADES  |
-        CAY       |
-        CARRIAGEWAY   | # typically appears as 'The Carriageway', a street noun
-        CAUSEWAY      | # a street noun
-        CENTRE        |
-        CONCORD       | # a street noun
-        CONCOURSE     |
-        CORNICHE      | # a street noun
-        CORONADO      | # a street noun
-        CORSO         | # a street noun
-        CIR       | CIRCLE   | CRCLE    |
-        CCT       | CRT      | CIR   | CIRCUIT  |
+        ALY?|ALLEY|
+        ARC|ARCADE|
+        BATTLEMENT|
+        BROADWATER|
+        BAYWAY|
+        BVD|BLVD?|BOULEVARDE?|
+        BND|BEND|
+        BL|BOWL|
+        BR|BRAE|
+        BROW|
+        CASCADES|
+        CAY|
+        CENTRE|
+        CONCOURSE|
+        CIR|CIRCLE|CRCLE|
+        CCT|CRT|CIR|CIRCUIT|
         CHASE|
-        CL        | CLOSE      |
-        CROSS     | CROSSOVER  | CROSSING |
-        CR?T      | COURT      |
-        CV        | COVE       |
-        CRES      | CRS        | CR    | CRESCENT  |
-        CREST     |
-        CROFT     |
-        DELL      |
-        DEVIATION |
-        DRIFTWAY  |
-        DR        | DRV        | DRIVE|
-        ENCLOSURE |
-        ENTRANCE  |
-        ESP       | ESPLANADE     |
-        EXP       | EXPW?Y        | EXPRESSWAY |
-        FAIRWAY   |
-        FENWAY    | # a street noun
-        FELLSWAY  | # a street noun
-        FW?Y      | FREEWAY|
-        GLADE     |
-        GRANGE    |
-        GREENWAY  | # a street noun
-        GLN       | GLEN|
-        GREENS?   | GRN|
-        GR        | GROVE|
-        HAVEN     |
-        HL        | HILL|
-        HWA?Y     | HIGHWAY|
-        HOLLOW    |
-        ISLE?     | IS         | ISLAND |
-        KEY       |
-        KINGSWAY  | # a street noun
-        KNOLL     |
-        LANTERNS  |
-        LANDING   |
-        LOOP      |
-        MAINBRACE | # a street noun
-        MAINSAIL  | # a street noun
-        MALL      | # a street noun
-        MEWS      |
-        OVERFLOW  |
-        OVAL      |
-        PASS      |
-        PASSAGE   | PSGE          | PSG|
-        PATH      |
-        PDE       | PARADE|
-        PK        | PARK|
-        PARKWAY   | PKWY|
-        PENINSULA |
-        PIERS     |
-        PIKE      |
-        PL        | PLACE|
-        PLZ       | PLAZA|
-        PORTICO   |
-        PROMENADE |
-        PT        | POINTE?|
-        RAMBLE    |
-        RAMPART   | # a street noun
-        RDG       | RIDGE         |
-        RETREAT   | # a street noun
-        RIDE      | RDE           |
-        RISE      | RSE           |
-        ROTUNDA   | # a street noun
-        RUN       |
-        RDY       | ROADWAY       |
-        ROW       |
-        SERPENTINE | # a street noun
-        STRAND    | # a street noun
-        SLIP      |
-        SQ        | SQUARE|
-        TCE       | TER           | TRCE     | TERRACE  |
-        TRL       | TRAIL|
-        TPKE      | TURNPIKE|
+        CL|CLOSE|
+        CROSS|CROSSOVER|CROSSING|
+        CR?T|COURT|
+        CV|COVE|
+        CRES|CRS|CR|CRESCENT|
+        CREST|
+        CROFT|
+        DELL|
+        DEVIATION|
+        DRIFTWAY|
+        DR|DRV|DRIVE|
+        ENCLOSURE|
+        ENTRANCE|
+        ESP|ESPLANADE|
+        EXP|EXPW?Y|EXPRESSWAY|
+        FAIRWAY|
+        FW?Y|FREEWAY|
+        GLADE|
+        GRANGE|
+        GLN|GLEN|
+        GREENS?|GRN|
+        GR|GROVE|
+        HAVEN|
+        HL|HILL|
+        HWA?Y|HIGHWAY|
+        HOLLOW|
+        ISLE?|IS|ISLAND|
+        KEY|
+        KNOLL|
+        LANTERNS|
+        LANDING|
+        LOOP|
+        MEWS|
+        OVERFLOW|
+        OVAL|
+        PASS|
+        PASSAGE|PSGE|PSG|
+        PATH|
+        PDE|PARADE|
+        PK|PARK|
+        PARKWAY|PKWY|
+        PENINSULA|
+        PIERS|
+        PIKE|
+        PL|PLACE|
+        PLZ|PLAZA|
+        PORTICO|
+        PROMENADE|
+        PT|POINTE?|
+        RAMBLE|
+        RDG|RIDGE|
+        RETREAT|
+        RIDE|RDE|
+        RISE|RSE|
+        RUN|
+        RDY|ROADWAY|
+        ROW|
+        SLIP|
+        SQ|SQUARE|
+        TCE|TER|TRCE|TERRACE|
+        TRL|TRAIL|
+        TPKE|TURNPIKE|
         TURN|
         THROUGHWAY|
-        VLG | VILLAGE |
-        WL?K   | WALK|
-        WY     | WAY| WYNDE |
+        VLG|VILLAGE|
+        WL?K|WALK|
+        WY|WAY|WYNDE|
         WAYS  # such as in 'The Five Ways'
-
         )\ /x  # note space separator needed at end of token
-
-
 };
 
 #------------------------------------------------------------------------------
@@ -741,51 +743,59 @@ q
 {
     suburb_prefix :
 
-        prefix   |
+        prefix  |
         /CAPE / |
-        /LAKE /
+        /LAKE / |
+        /MOUNT|MT /
 
-   # suburb: prefix? any_word suburb_word(0..2)
-
-    suburb:
-
-        # such as Upper Ferntree Gully
-        suburb_prefix any_word suburb_word(?)
+    suburb: 
+        any_word /BY THE SEA /
         {
-            if ( $item[3][0]  )
-            {
-                $return = "$item[1]$item[2]$item[3][0]"
-            }
-            else
-            {
-                $return = "$item[1]$item[2]"
-            }
+               $return = "$item[1]$item[2]"
         }
         |
-
-        # such as  Washington Valley, Lane Cove West
-        any_word suburb_word(0..2)
+        /LAND O LAKES /                
+        |
+        # such as  Washington Valley, Lane Cove West, Little Egg Harbour Township
+        suburb_prefix(?) any_word suburb_word(0..2)
         {
-            if ( $item[2][0] and $item[2][1]  )
+            if ( $item[1][0] )
             {
-               $return = "$item[1]$item[2][0]$item[2][1]"
-            }
-            elsif ( $item[2][0]  )
-            {
-               $return = "$item[1]$item[2][0]"
+                if ($item[3][0] and $item[3][1])
+                {
+                    $return = "$item[1][0]$item[2]$item[3][0]$item[3][1]"
+                }
+                elsif ( $item[3][0] )
+                {
+                   $return = "$item[1][0]$item[2]$item[3][0]"
+                }
+                else
+                {
+                   $return = "$item[1][0]$item[2]"
+                }
             }
             else
             {
-              $return = $item[1]
+               if ($item[3][0] and $item[3][1])
+                {
+                    $return = "$item[2]$item[3][0]$item[3][1]"
+                }
+                elsif ( $item[3][0] )
+                {
+                   $return = "$item[2]$item[3][0]"
+                }
+                else
+                {
+                   $return = "$item[2]"
+                }
             }
         }
         |
         # such as Kippa-ring or Brighton-Le-Sands
-        /[A-Z]{2,}-[A-Z]{2,}(-[A-Z]{2,})?/
+        /[A-Z]{2,}-[A-Z]{2,}(-[A-Z]{2,})? /
 
     suburb_word: ...!subcountry any_word
 };
-
 #------------------------------------------------------------------------------
 my $common_terms =
 q
@@ -809,7 +819,6 @@ q
         WEST
         )\ /x
 
-
    abbrev_direction:
         /(
         N  |
@@ -914,7 +923,7 @@ sub _create
     $grammar .= $post_box_rule;
     $grammar .= $road_box_rule;
     $grammar .= $non_matching_rule;
-    $grammar .= $sub_property_identifier;
+    $grammar .= $sub_property;
     $grammar .= $property_identifier;
     $grammar .= $property_name;
     $grammar .= $post_box;
@@ -998,7 +1007,6 @@ sub _create
     $grammar .= $non_matching;
 
     return($grammar);
-
 }
 #-------------------------------------------------------------------------------
 # Some sub countries contain descriptive text, such as
